@@ -30,7 +30,7 @@ from pulpcore.plugin.repo_version_utils import (
     validate_version_paths,
 )
 
-from pulp_rpm.app.constants import CHECKSUM_CHOICES, COMPRESSION_CHOICES
+from pulp_rpm.app.constants import CHECKSUM_CHOICES, COMPRESSION_CHOICES, LAYOUT_CHOICES
 from pulp_rpm.app.downloaders import RpmDownloader, RpmFileDownloader, UlnDownloader
 from pulp_rpm.app.exceptions import DistributionTreeConflict
 from pulp_rpm.app.models import (
@@ -47,7 +47,7 @@ from pulp_rpm.app.models import (
     RpmPackageSigningService,
     UpdateRecord,
 )
-from pulp_rpm.app.shared_utils import urlpath_sanitize
+from pulp_rpm.app.shared_utils import urlpath_sanitize, annotate_with_age
 
 log = getLogger(__name__)
 
@@ -208,6 +208,8 @@ class RpmRepository(Repository, AutoAddObjPermsMixin):
         repo_config (JSON): repo configuration that will be served by distribution
         compression_type(pulp_rpm.app.constants.COMPRESSION_TYPES):
             Compression type to use for metadata files.
+        layout(pulp_rpm.app.constants.LAYOUT_TYPES):
+            How to layout the package files within the publication (flat, nested, etc.)
     """
 
     TYPE = "rpm"
@@ -233,13 +235,14 @@ class RpmRepository(Repository, AutoAddObjPermsMixin):
         RpmPackageSigningService, on_delete=models.SET_NULL, null=True
     )
     package_signing_fingerprint = models.TextField(null=True, max_length=40)
-    original_checksum_types = models.JSONField(default=dict)
+    original_checksum_types = models.JSONField(default=dict)  # DEPRECATED, remove in 3.29+
     last_sync_details = models.JSONField(default=dict)
     retain_package_versions = models.PositiveIntegerField(default=0)
 
     autopublish = models.BooleanField(default=False)
     checksum_type = models.TextField(null=True, choices=CHECKSUM_CHOICES)
     compression_type = models.TextField(null=True, choices=COMPRESSION_CHOICES)
+    layout = models.TextField(null=True, choices=LAYOUT_CHOICES)
     metadata_checksum_type = models.TextField(null=True, choices=CHECKSUM_CHOICES)
     package_checksum_type = models.TextField(null=True, choices=CHECKSUM_CHOICES)
     repo_config = models.JSONField(default=dict)
@@ -267,6 +270,7 @@ class RpmRepository(Repository, AutoAddObjPermsMixin):
                 },
                 repo_config=self.repo_config,
                 compression_type=self.compression_type,
+                layout=self.layout,
             )
 
     @property
@@ -450,13 +454,11 @@ class RpmRepository(Repository, AutoAddObjPermsMixin):
             # django-managed and would be difficult to share.
             #
             # Instead we have to do the filtering manually.
-            nonmodular_packages = (
-                Package.objects.with_age()
-                .filter(
+            nonmodular_packages = annotate_with_age(
+                Package.objects.filter(
                     pk__in=new_version.content.filter(pulp_type=Package.get_pulp_type()),
                     is_modular=False,  # don't want to filter out modular RPMs
-                )
-                .only("pk")
+                ).only("pk")
             )
 
             old_packages = []
@@ -487,9 +489,7 @@ class RpmRepository(Repository, AutoAddObjPermsMixin):
 
         incoming_disttrees = new_version.content.filter(pulp_type=disttree_pulp_type)
         if len(incoming_disttrees) != 1:
-            raise DistributionTreeConflict(
-                _("More than one distribution tree cannot be added to a " "repository version.")
-            )
+            raise DistributionTreeConflict()
 
 
 class RpmPublication(Publication, AutoAddObjPermsMixin):
@@ -502,6 +502,7 @@ class RpmPublication(Publication, AutoAddObjPermsMixin):
     compression_type = models.TextField(null=True, choices=COMPRESSION_CHOICES)
     metadata_checksum_type = models.TextField(choices=CHECKSUM_CHOICES)
     package_checksum_type = models.TextField(choices=CHECKSUM_CHOICES)
+    layout = models.TextField(null=True, choices=LAYOUT_CHOICES)
     repo_config = models.JSONField(default=dict)
 
     class Meta:
@@ -529,10 +530,28 @@ class RpmDistribution(Distribution, AutoAddObjPermsMixin):
             repository, publication = self.get_repository_and_publication()
             if not publication:
                 return
+
+            # "Where content will be retrieved from" comes first from CONTENT_ORIGIN.
+            # If that's not set, use a specified baseurl.
+            # If *that* isn't set - fail, we can't build a config.repo because we
+            # don't have enough information to set the baseurl correctly.
+            origin = (
+                settings.CONTENT_ORIGIN
+                if settings.CONTENT_ORIGIN
+                else publication.repo_config.get("baseurl")
+            )
+            if not origin:
+                return Response(
+                    status=404,
+                    reason=_(
+                        "Cannot auto-generate config.repo when CONTENT_ORIGIN is not set and "
+                        "no baseurl specified."
+                    ),
+                )
             if settings.DOMAIN_ENABLED:
                 base_url = "{}/".format(
                     urlpath_sanitize(
-                        settings.CONTENT_ORIGIN,
+                        origin,
                         settings.CONTENT_PATH_PREFIX,
                         self.pulp_domain.name,
                         self.base_path,
@@ -541,7 +560,7 @@ class RpmDistribution(Distribution, AutoAddObjPermsMixin):
             else:
                 base_url = "{}/".format(
                     urlpath_sanitize(
-                        settings.CONTENT_ORIGIN,
+                        origin,
                         settings.CONTENT_PATH_PREFIX,
                         self.base_path,
                     )
