@@ -1,4 +1,6 @@
 from .publishing import publish  # noqa
 from .synchronizing import synchronize  # noqa
+from .signing import sign_and_create  # noqa
 from .copy import copy_content  # noqa
 from .comps import upload_comps  # noqa
+from .prune import prune_packages  # noqa
