@@ -87,7 +87,6 @@ from pulp_rpm.app.modulemd import parse_modular
 
 from pulp_rpm.app.comps import strdict_to_dict, dict_digest
 from pulp_rpm.app.kickstart.treeinfo import PulpTreeInfo, TreeinfoData
-from pulp_rpm.app.metadata_parsing import MetadataParser
 from pulp_rpm.app.shared_utils import (
     is_previous_version,
     get_sha256,
@@ -805,8 +804,6 @@ class RpmFirstStage(Stage):
                         run_repomdrecord_download(record.type, record.location_href, downloader)
                     )
 
-                self.repository.original_checksum_types = checksum_types
-
                 try:
                     for future in asyncio.as_completed(list(repomd_downloaders.values())):
                         name, location_href, result = await future
@@ -906,13 +903,17 @@ class RpmFirstStage(Stage):
 
     async def parse_repository_metadata(self, repomd, metadata_results):
         """Parse repository metadata."""
-        needed_metadata = set(PACKAGE_REPODATA) - set(metadata_results.keys())
-
-        if needed_metadata:
+        if "primary" not in metadata_results.keys():
             raise FileNotFoundError(
-                _("XML file(s): {filenames} not found").format(filenames=", ".join(needed_metadata))
+                "Repository doesn't contain required metadata file 'primary.xml'"
             )
 
+        if "filelists" not in metadata_results.keys():
+            log.warn("Repository doesn't contain metadata file 'filelists.xml'")
+
+        if "other" not in metadata_results.keys():
+            log.warn("Repository doesn't contain metadata file 'other.xml'")
+
         await self.parse_distribution_tree()
 
         # modularity-parsing MUST COME BEFORE package-parsing!
@@ -932,9 +933,9 @@ class RpmFirstStage(Stage):
 
         # **Now** we can successfully parse package-metadata
         await self.parse_packages(
-            metadata_results["primary"],
-            metadata_results["filelists"],
-            metadata_results["other"],
+            metadata_results.get("primary"),
+            metadata_results.get("filelists"),
+            metadata_results.get("other"),
             modulemd_list=modulemd_list,
         )
 
@@ -1172,8 +1173,10 @@ class RpmFirstStage(Stage):
 
     async def parse_packages(self, primary_xml, filelists_xml, other_xml, modulemd_list=None):
         """Parse packages from the remote repository."""
-        parser = MetadataParser.from_metadata_files(
-            primary_xml.path, filelists_xml.path, other_xml.path
+        parser = cr.RepositoryReader.from_metadata_files(
+            primary_xml.path,
+            filelists_xml.path if filelists_xml else None,
+            other_xml.path if other_xml else None,
         )
 
         # skip SRPM if defined
@@ -1272,7 +1275,7 @@ class RpmFirstStage(Stage):
 
         # Ew, callback-based API, gross. The streaming API doesn't support optionally
         # specifying particular files yet so we have to use the old way.
-        parser.for_each_pkg_primary(verification_and_skip_callback)
+        cr.xml_parse_primary(primary_xml.path, pkgcb=verification_and_skip_callback, do_files=False)
 
         # Go through the package lists, sort them descending by EVR, ignore the first N and then
         # add the remaining ones to the skip list.
@@ -1308,7 +1311,7 @@ class RpmFirstStage(Stage):
             "total": total_packages,
         }
         async with ProgressReport(**progress_data) as packages_pb:
-            for pkg in parser.as_iterator():
+            for pkg in parser.iter_packages():
                 pkg_nevra = pkg.nevra()
                 # Skip over packages (retention feature, skip_types feature)
                 if package_skip_nevras and pkg_nevra in package_skip_nevras:
@@ -1325,15 +1328,21 @@ class RpmFirstStage(Stage):
                 url = urlpath_sanitize(base_url, package.location_href)
                 del pkg  # delete it as soon as we're done with it
 
-                store_package_for_mirroring(self.repository, package.pkgId, package.location_href)
+                # Location_href is not a property of the Package in isolation [0], and Pulp has
+                # a well defined way of generating the layout/locations on publication time.
+                # We only need to use the original location_href for metadata mirroring
+                # [0] https://github.com/pulp/pulp_rpm/issues/2580
+                original_location_href = package.location_href
+                package.location_href = package.filename
+                store_package_for_mirroring(self.repository, package.pkgId, original_location_href)
+
                 artifact = Artifact(size=package.size_package)
                 checksum_type = getattr(CHECKSUM_TYPES, package.checksum_type.upper())
                 setattr(artifact, checksum_type, package.pkgId)
-                filename = os.path.basename(package.location_href)
                 da = DeclarativeArtifact(
                     artifact=artifact,
                     url=url,
-                    relative_path=filename,
+                    relative_path=package.location_href,
                     remote=self.remote,
                     deferred_download=self.deferred_download,
                 )
