@@ -68,6 +68,14 @@ class PackageViewSet(SingleArtifactContentUploadViewSet):
                     "has_required_repo_perms_on_upload:rpm.view_rpmrepository",
                 ],
             },
+            {
+                "action": ["set_label", "unset_label"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": [
+                    "has_model_or_domain_perms:core.manage_content_labels",
+                ],
+            },
         ],
         "queryset_scoping": {"function": "scope_queryset"},
     }
