@@ -12,8 +12,6 @@ import requests
 import xmltodict
 import dictdiffer
 
-from pulp_smash.pulp3.utils import gen_repo, gen_distribution
-
 from pulp_rpm.tests.functional.constants import (
     RPM_ALT_LAYOUT_FIXTURE_URL,
     RPM_COMPLEX_FIXTURE_URL,
@@ -29,8 +27,7 @@ from pulp_rpm.tests.functional.constants import (
     RPM_UNSIGNED_FIXTURE_URL,
     SRPM_UNSIGNED_FIXTURE_URL,
 )
-from pulp_rpm.tests.functional.utils import gen_rpm_remote, download_and_decompress_file
-from pulp_rpm.tests.functional.utils import set_up_module as setUpModule  # noqa:F401
+from pulp_rpm.tests.functional.utils import download_and_decompress_file
 
 from pulpcore.client.pulp_rpm import RpmRepositorySyncURL, RpmRpmPublication
 from pulpcore.client.pulp_rpm.exceptions import ApiException
@@ -77,13 +74,14 @@ class TestPublishWithUnsignedRepoSyncedImmediate:
     @pytest.mark.parallel
     def test_publish_with_compression_types(
         self,
+        distribution_base_url,
         compression_type,
         compression_ext,
         rpm_unsigned_repo_immediate,
         rpm_publication_api,
-        gen_object_with_cleanup,
         rpm_distribution_api,
         monitor_task,
+        rpm_distribution_factory,
     ):
         """Sync and publish an RPM repository w/ zstd compression and verify it exists."""
         # 1. Publish and distribute
@@ -94,22 +92,24 @@ class TestPublishWithUnsignedRepoSyncedImmediate:
         created_resources = monitor_task(publish_response.task).created_resources
         publication_href = created_resources[0]
 
-        body = gen_distribution(publication=publication_href)
-        distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+        distribution = rpm_distribution_factory(publication=publication_href)
 
         # 2. Check "primary", "filelists", "other", "updateinfo" have correct compression ext
-        for md_type, md_href in self.get_repomd_metadata_urls(distribution.base_url).items():
+        for md_type, md_href in self.get_repomd_metadata_urls(
+            distribution_base_url(distribution.base_url)
+        ).items():
             if md_type in ("primary", "filelists", "other", "updateinfo"):
                 assert md_href.endswith(compression_ext)
 
     @pytest.mark.parallel
     def test_validate_no_checksum_tag(
         self,
+        distribution_base_url,
         rpm_unsigned_repo_immediate,
         rpm_publication_api,
-        gen_object_with_cleanup,
         rpm_distribution_api,
         monitor_task,
+        rpm_distribution_factory,
     ):
         """Sync and publish an RPM repository and verify the checksum.
 
@@ -121,13 +121,14 @@ class TestPublishWithUnsignedRepoSyncedImmediate:
         created_resources = monitor_task(publish_response.task).created_resources
         publication_href = created_resources[0]
 
-        body = gen_distribution(publication=publication_href)
-        distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+        distribution = rpm_distribution_factory(publication=publication_href)
 
         # 2. check the tag 'sum' is not present in updateinfo.xml
-        update_xml_url = self.get_repomd_metadata_urls(distribution.base_url)["updateinfo"]
+        update_xml_url = self.get_repomd_metadata_urls(
+            distribution_base_url(distribution.base_url)
+        )["updateinfo"]
         update_xml = download_and_decompress_file(
-            os.path.join(distribution.base_url, update_xml_url)
+            os.path.join(distribution_base_url(distribution.base_url), update_xml_url)
         )
         update_info_content = ElementTree.fromstring(update_xml)
 
@@ -141,7 +142,7 @@ class TestPublishWithUnsignedRepoSyncedImmediate:
 
         Example:
             ```
-            >>> get_repomd_metadata_urls(distribution.base_url)
+            >>> get_repomd_metadata_urls(distribution_base_url(distribution.base_url))
             {
                 "primary": "repodata/.../primary.xml.gz",
                 "filelists": "repodata/.../filelists.xml.gz",
@@ -223,15 +224,91 @@ def test_publish_references_update(assert_created_publication):
     assert_created_publication(RPM_REFERENCES_UPDATEINFO_URL)
 
 
-@pytest.mark.parametrize("repo_url", [RPM_COMPLEX_FIXTURE_URL, RPM_MODULAR_FIXTURE_URL])
-def test_complex_repo_core_metadata(
-    repo_url,
+def get_metadata_content_helper(base_url, repomd_elem, meta_type):
+    """Return the text contents of metadata file.
+
+    Provided a url, a repomd root element, and a metadata type, locate the metadata
+    file's location href, download it from the provided url, un-gzip it, parse it, and
+    return the root element node.
+
+    Don't use this with large repos because it will blow up.
+    """
+    # <ns0:repomd xmlns:ns0="http://linux.duke.edu/metadata/repo">
+    #     <ns0:data type="primary">
+    #         <ns0:checksum type="sha256">[…]</ns0:checksum>
+    #         <ns0:location href="repodata/[…]-primary.xml.gz" />
+    #         …
+    #     </ns0:data>
+    #     …
+    xpath = "{{{}}}data".format(RPM_NAMESPACES["metadata/repo"])
+    data_elems = [elem for elem in repomd_elem.findall(xpath) if elem.get("type") == meta_type]
+    if not data_elems:
+        return None
+
+    xpath = "{{{}}}location".format(RPM_NAMESPACES["metadata/repo"])
+    location_href = data_elems[0].find(xpath).get("href")
+
+    return download_and_decompress_file(os.path.join(base_url, location_href))
+
+
+@pytest.mark.parametrize("layout", ["flat", "nested_alphabetically"])
+def test_repo_layout(
+    layout,
     init_and_sync,
     rpm_publication_api,
     gen_object_with_cleanup,
     rpm_distribution_api,
+    rpm_distribution_factory,
     monitor_task,
     delete_orphans_pre,
+    tmpdir,
+    wget_recursive_download_on_host,
+):
+    """Test that using the "layout" option for publication produces the correct package layouts"""
+
+    # create repo and remote
+    repo, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
+
+    # publish
+    publish_data = RpmRpmPublication(repository=repo.pulp_href, layout=layout)
+    publish_response = rpm_publication_api.create(publish_data)
+    created_resources = monitor_task(publish_response.task).created_resources
+    publication_href = created_resources[0]
+
+    # distribute
+    distribution = rpm_distribution_factory(publication=publication_href)
+
+    # Download and parse the metadata.
+    repomd = ElementTree.fromstring(
+        requests.get(os.path.join(distribution.base_url, "repodata/repomd.xml")).text
+    )
+
+    # Convert the metadata into a more workable form and then compare.
+    primary = get_metadata_content_helper(distribution.base_url, repomd, "primary")
+
+    packages = xmltodict.parse(primary, dict_constructor=collections.OrderedDict)["metadata"][
+        "package"
+    ]
+
+    for package in packages:
+        if layout == "flat":
+            assert package["location"]["@href"].startswith("Packages/{}".format(package["name"][0]))
+        elif layout == "nested_alphabetically":
+            assert package["location"]["@href"].startswith(
+                "Packages/{}/".format(package["name"][0])
+            )
+
+
+@pytest.mark.parametrize("repo_url", [RPM_COMPLEX_FIXTURE_URL, RPM_MODULAR_FIXTURE_URL])
+def test_complex_repo_core_metadata(
+    distribution_base_url,
+    repo_url,
+    init_and_sync,
+    rpm_publication_api,
+    rpm_distribution_api,
+    monitor_task,
+    delete_orphans_pre,
+    rpm_distribution_factory,
 ):
     """Test the "complex" fixture that covers more of the metadata cases.
 
@@ -250,8 +327,7 @@ def test_complex_repo_core_metadata(
     publication_href = created_resources[0]
 
     # distribute
-    body = gen_distribution(publication=publication_href)
-    distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+    distribution = rpm_distribution_factory(publication=publication_href)
 
     # Download and parse the metadata.
     original_repomd = ElementTree.fromstring(
@@ -259,48 +335,26 @@ def test_complex_repo_core_metadata(
     )
 
     reproduced_repomd = ElementTree.fromstring(
-        requests.get(os.path.join(distribution.base_url, "repodata/repomd.xml")).text
+        requests.get(
+            os.path.join(distribution_base_url(distribution.base_url), "repodata/repomd.xml")
+        ).text
     )
 
-    def get_metadata_content(base_url, repomd_elem, meta_type):
-        """Return the text contents of metadata file.
-
-        Provided a url, a repomd root element, and a metadata type, locate the metadata
-        file's location href, download it from the provided url, un-gzip it, parse it, and
-        return the root element node.
-
-        Don't use this with large repos because it will blow up.
-        """
-        # <ns0:repomd xmlns:ns0="http://linux.duke.edu/metadata/repo">
-        #     <ns0:data type="primary">
-        #         <ns0:checksum type="sha256">[…]</ns0:checksum>
-        #         <ns0:location href="repodata/[…]-primary.xml.gz" />
-        #         …
-        #     </ns0:data>
-        #     …
-        xpath = "{{{}}}data".format(RPM_NAMESPACES["metadata/repo"])
-        data_elems = [elem for elem in repomd_elem.findall(xpath) if elem.get("type") == meta_type]
-        if not data_elems:
-            return None
-
-        xpath = "{{{}}}location".format(RPM_NAMESPACES["metadata/repo"])
-        location_href = data_elems[0].find(xpath).get("href")
-
-        return download_and_decompress_file(os.path.join(base_url, location_href))
-
     # Convert the metadata into a more workable form and then compare.
     for metadata_file in ["primary", "filelists", "other"]:
-        original_metadata = get_metadata_content(repo_url, original_repomd, metadata_file)
-        generated_metadata = get_metadata_content(
-            distribution.base_url, reproduced_repomd, metadata_file
+        original_metadata = get_metadata_content_helper(repo_url, original_repomd, metadata_file)
+        generated_metadata = get_metadata_content_helper(
+            distribution_base_url(distribution.base_url), reproduced_repomd, metadata_file
         )
 
         _compare_xml_metadata_file(original_metadata, generated_metadata, metadata_file)
 
     # =================
 
-    original_modulemds = get_metadata_content(repo_url, original_repomd, "modules")
-    generated_modulemds = get_metadata_content(distribution.base_url, reproduced_repomd, "modules")
+    original_modulemds = get_metadata_content_helper(repo_url, original_repomd, "modules")
+    generated_modulemds = get_metadata_content_helper(
+        distribution_base_url(distribution.base_url), reproduced_repomd, "modules"
+    )
 
     assert bool(original_modulemds) == bool(generated_modulemds)
 
@@ -314,9 +368,9 @@ def test_complex_repo_core_metadata(
     # ===================
 
     # TODO: make this deeper
-    original_updateinfo = get_metadata_content(repo_url, original_repomd, "updateinfo")
-    generated_updateinfo = get_metadata_content(
-        distribution.base_url, reproduced_repomd, "updateinfo"
+    original_updateinfo = get_metadata_content_helper(repo_url, original_repomd, "updateinfo")
+    generated_updateinfo = get_metadata_content_helper(
+        distribution_base_url(distribution.base_url), reproduced_repomd, "updateinfo"
     )
     assert bool(original_updateinfo) == bool(generated_updateinfo)
 
@@ -411,12 +465,15 @@ def _compare_xml_metadata_file(original_metadata_text, generated_metadata_text,
 @pytest.mark.parallel
 @pytest.mark.parametrize("mirror", [True, False], ids=["mirror", "standard"])
 def test_distribution_tree_metadata_publish(
+    distribution_base_url,
     mirror,
-    gen_object_with_cleanup,
     rpm_repository_api,
     rpm_rpmremote_api,
     rpm_distribution_api,
     monitor_task,
+    rpm_rpmremote_factory,
+    rpm_repository_factory,
+    rpm_distribution_factory,
 ):
     """Test the "complex" fixture that covers more of the metadata cases.
 
@@ -427,10 +484,8 @@ def test_distribution_tree_metadata_publish(
     from configparser import ConfigParser
 
     # 1. create repo and remote
-    repo = gen_object_with_cleanup(rpm_repository_api, gen_repo(autopublish=not mirror))
-
-    body = gen_rpm_remote(RPM_KICKSTART_FIXTURE_URL, policy="on_demand")
-    remote = gen_object_with_cleanup(rpm_rpmremote_api, body)
+    repo = rpm_repository_factory(autopublish=not mirror)
+    remote = rpm_rpmremote_factory(url=RPM_KICKSTART_FIXTURE_URL, policy="on_demand")
 
     # 2, 3. Sync and publish
     repository_sync_data = RpmRepositorySyncURL(remote=remote.pulp_href, mirror=mirror)
@@ -440,12 +495,13 @@ def test_distribution_tree_metadata_publish(
 
     publication_href = [r for r in created_resources if "publication" in r][0]
 
-    body = gen_distribution(publication=publication_href)
-    distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+    distribution = rpm_distribution_factory(publication=publication_href)
 
     # 4. Download and parse the metadata.
     original_treeinfo = requests.get(os.path.join(RPM_KICKSTART_FIXTURE_URL, ".treeinfo")).text
-    generated_treeinfo = requests.get(os.path.join(distribution.base_url, ".treeinfo")).text
+    generated_treeinfo = requests.get(
+        os.path.join(distribution_base_url(distribution.base_url), ".treeinfo")
+    ).text
 
     config = ConfigParser()
     config.optionxform = str  # by default it will cast keys to lower case
@@ -495,16 +551,22 @@ def test_distribution_tree_metadata_publish(
             continue
 
         checksum_type, checksum = checksum.split(":")
-        assert requests.get(os.path.join(distribution.base_url, path)).status_code == 200
+        assert (
+            requests.get(
+                os.path.join(distribution_base_url(distribution.base_url), path)
+            ).status_code
+            == 200
+        )
 
 
 @pytest.fixture
 def get_checksum_types(
+    distribution_base_url,
     init_and_sync,
     rpm_publication_api,
-    gen_object_with_cleanup,
     rpm_distribution_api,
     monitor_task,
+    rpm_distribution_factory,
 ):
     """Sync and publish an RPM repository."""
 
@@ -527,11 +589,12 @@ def get_checksum_types(
         created_resources = monitor_task(publish_response.task).created_resources
         publication_href = created_resources[0]
 
-        body = gen_distribution(publication=publication_href)
-        distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+        distribution = rpm_distribution_factory(publication=publication_href)
 
         repomd = ElementTree.fromstring(
-            requests.get(os.path.join(distribution.base_url, "repodata/repomd.xml")).text
+            requests.get(
+                os.path.join(distribution_base_url(distribution.base_url), "repodata/repomd.xml")
+            ).text
         )
 
         data_xpath = "{{{}}}data".format(RPM_NAMESPACES["metadata/repo"])
@@ -547,7 +610,9 @@ def get_checksum_types(
                 location_xpath = "{{{}}}location".format(RPM_NAMESPACES["metadata/repo"])
                 primary_href = data_elem.find(location_xpath).get("href")
                 primary = ElementTree.fromstring(
-                    download_and_decompress_file(os.path.join(distribution.base_url, primary_href))
+                    download_and_decompress_file(
+                        os.path.join(distribution_base_url(distribution.base_url), primary_href)
+                    )
                 )
                 package_checksum_xpath = "{{{}}}checksum".format(RPM_NAMESPACES["metadata/common"])
                 package_xpath = "{{{}}}package".format(RPM_NAMESPACES["metadata/common"])
@@ -605,17 +670,14 @@ def test_publish_with_unsupported_checksum_type(rpm_unsigned_repo_on_demand, rpm
 
 
 @pytest.mark.parallel
-def test_on_demand_unspecified_checksum_types(get_checksum_types):
+def test_unspecified_checksum_types(get_checksum_types):
     """Sync and publish an RPM repository and verify the checksum types."""
     repomd_checksum_types, primary_checksum_types = get_checksum_types(
         fixture_url=RPM_SHA512_FIXTURE_URL, policy="on_demand"
     )
 
     for repomd_type, repomd_checksum_type in repomd_checksum_types.items():
-        # hack to account for the fact that our md5 and sha512 repos use md5 and sha256
-        # checksums for all metadata *except* updateinfo :(
-        if not repomd_type == "updateinfo":
-            assert repomd_checksum_type == "sha512"
+        assert repomd_checksum_type == "sha256"
 
     for package, package_checksum_type in primary_checksum_types.items():
         # since none of the packages in question have sha512 checksums, the
@@ -623,23 +685,6 @@ def test_on_demand_unspecified_checksum_types(get_checksum_types):
         assert package_checksum_type == "sha512"
 
 
-@pytest.mark.parallel
-def test_immediate_unspecified_checksum_types(get_checksum_types):
-    """Sync and publish an RPM repository and verify the checksum types."""
-    repomd_checksum_types, primary_checksum_types = get_checksum_types(
-        fixture_url=RPM_SHA512_FIXTURE_URL, policy="immediate"
-    )
-
-    for repomd_type, repomd_checksum_type in repomd_checksum_types.items():
-        # hack to account for the fact that our md5 and sha512 repos use md5 and sha256
-        # checksums for all metadata *except* updateinfo :(
-        if not repomd_type == "updateinfo":
-            assert repomd_checksum_type == "sha512"
-
-    for package, package_checksum_type in primary_checksum_types.items():
-        assert package_checksum_type == "sha512"
-
-
 def test_on_demand_specified_package_checksum_type(get_checksum_types, delete_orphans_pre):
     """Sync and publish an RPM repository and verify the checksum types."""
     repomd_checksum_types, primary_checksum_types = get_checksum_types(
@@ -729,9 +774,12 @@ def test_immediate_specified_metadata_and_package_checksum_type(get_checksum_typ
 
 
 @pytest.mark.parallel
-def test_directory_layout_distribute_with_modules(generate_distribution):
+def test_directory_layout_distribute_with_modules(
+    distribution_base_url,
+    generate_distribution,
+):
     """Ensure no more files or folders are present when distribute repository with modules."""
-    distribution = generate_distribution(RPM_MODULAR_FIXTURE_URL)
+    distribution = distribution_base_url(generate_distribution(RPM_MODULAR_FIXTURE_URL))
     parser = etree.XMLParser(recover=True)
     repository = etree.fromstring(requests.get(distribution).text, parser=parser)
     # Get links from repository HTML
@@ -750,9 +798,12 @@ def test_directory_layout_distribute_with_modules(generate_distribution):
 
 
 @pytest.mark.parallel
-def test_directory_layout_distribute_with_treeinfo(generate_distribution):
+def test_directory_layout_distribute_with_treeinfo(
+    generate_distribution,
+    distribution_base_url,
+):
     """Ensure no more files or folders are present when distribute repository with treeinfo."""
-    distribution = generate_distribution(RPM_KICKSTART_FIXTURE_URL)
+    distribution = distribution_base_url(generate_distribution(RPM_KICKSTART_FIXTURE_URL))
     parser = etree.XMLParser(recover=True)
     repository = etree.fromstring(requests.get(distribution).text, parser=parser)
     # Get links from repository HTML
@@ -775,10 +826,10 @@ def test_directory_layout_distribute_with_treeinfo(generate_distribution):
 @pytest.fixture(scope="class")
 def generate_distribution(
     init_and_sync,
-    gen_object_with_cleanup,
     rpm_distribution_api,
     rpm_publication_api,
     monitor_task,
+    rpm_distribution_factory,
 ):
     def _generate_distribution(url=None):
         """Sync and publish an RPM repository.
@@ -803,9 +854,8 @@ def generate_distribution(
         created_resources = monitor_task(publish_response.task).created_resources
         publication_href = created_resources[0]
 
-        body = gen_distribution(publication=publication_href)
-        distribution = gen_object_with_cleanup(rpm_distribution_api, body)
+        distribution = rpm_distribution_factory(publication=publication_href)
 
-        return distribution.to_dict()["base_url"]
+        return distribution.base_url
 
     return _generate_distribution
