@@ -3,21 +3,15 @@
 import pytest
 from random import choice
 
+from urllib.parse import urljoin
 import dictdiffer
 import requests
+from pathlib import Path
 from django.conf import settings
 from django.utils.dateparse import parse_datetime
+import createrepo_c as cr
 
 from pulpcore.tests.functional.utils import PulpTaskError
-from pulp_smash.pulp3.utils import (
-    gen_repo,
-    get_added_content_summary,
-    get_added_content,
-    get_content,
-    get_content_summary,
-    get_removed_content,
-    wget_download_on_host,
-)
 
 from pulp_rpm.tests.functional.constants import (
     AMAZON_MIRROR,
@@ -71,55 +65,51 @@ from pulp_rpm.tests.functional.constants import (
     RPM_MODULEMDS_DATA,
     RPM_ZSTD_METADATA_FIXTURE_URL,
 )
-from pulp_rpm.tests.functional.utils import gen_rpm_remote
-from pulp_rpm.tests.functional.utils import set_up_module as setUpModule  # noqa:F401
-
 from pulpcore.client.pulp_rpm import RpmRepositorySyncURL
 from pulpcore.client.pulp_rpm.exceptions import ApiException
 
 
 @pytest.mark.parallel
-def test_sync(init_and_sync):
+def test_sync(init_and_sync, get_content_summary):
     """Sync repositories with the rpm plugin."""
     # Create a remote (default) and empty repository
+    # Then Assert that it's synced properly
     repository, remote = init_and_sync()
-
-    # Assert that it's synced properly
-    latest_version_href = repository.latest_version_href
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    original_latest_version_href = repository.latest_version_href
+    assert get_content_summary(repository)["present"] == RPM_FIXTURE_SUMMARY
 
     # Sync the same repository again
+    # Then assert that the repository has not changed, the latest version stays the same
     repository, _ = init_and_sync(repository=repository, remote=remote)
-
-    # Assert that the repository has not changed, the latest version stays the same
-    assert latest_version_href == repository.latest_version_href
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    assert original_latest_version_href == repository.latest_version_href
+    assert get_content_summary(repository)["present"] == RPM_FIXTURE_SUMMARY
 
 
 @pytest.mark.parallel
-def test_sync_zstd(init_and_sync):
+def test_sync_zstd(init_and_sync, get_content_summary):
     """Test syncing non-gzip metadata."""
     repository, _ = init_and_sync(url=RPM_ZSTD_METADATA_FIXTURE_URL)
 
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_FIXTURE_SUMMARY
 
 
 @pytest.mark.parallel
-def test_sync_local(init_and_sync, tmpdir):
+def test_sync_local(init_and_sync, tmpdir, wget_recursive_download_on_host):
     """Test syncing from the local filesystem."""
-    wget_download_on_host(RPM_UNSIGNED_FIXTURE_URL, str(tmpdir))
+    wget_recursive_download_on_host(RPM_UNSIGNED_FIXTURE_URL, str(tmpdir))
     init_and_sync(url=f"file://{tmpdir}/rpm-unsigned/")
 
 
 @pytest.mark.parallel
-def test_sync_from_valid_mirror_list_feed(init_and_sync):
+def test_sync_from_valid_mirror_list_feed(init_and_sync, get_content_summary):
     """Sync RPM content from a mirror list feed which contains a valid remote URL."""
     repository, _ = init_and_sync(url=RPM_MIRROR_LIST_GOOD_FIXTURE_URL)
 
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_FIXTURE_SUMMARY
 
 
 @pytest.mark.parallel
@@ -134,20 +124,21 @@ def test_sync_from_invalid_mirror_list_feed(init_and_sync):
     with pytest.raises(PulpTaskError) as exc:
         init_and_sync(url=RPM_MIRROR_LIST_BAD_FIXTURE_URL)
 
-    assert "An invalid remote URL was provided" in exc.value.task.to_dict()["error"]["description"]
+    assert "An invalid remote URL was provided" in exc.value.task.error["description"]
 
 
 @pytest.mark.parallel
-def test_sync_modular(init_and_sync):
+def test_sync_modular(init_and_sync, get_content_summary):
     """Sync RPM modular content."""
     repository, _ = init_and_sync(url=RPM_MODULAR_FIXTURE_URL)
 
-    assert get_content_summary(repository.to_dict()) == RPM_MODULAR_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_MODULAR_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_MODULAR_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_MODULAR_FIXTURE_SUMMARY
 
 
 @pytest.mark.parallel
-def test_checksum_constraint(init_and_sync):
+def test_checksum_constraint(init_and_sync, get_content_summary):
     """Verify checksum constraint test case.
 
     Do the following:
@@ -161,14 +152,14 @@ def test_checksum_constraint(init_and_sync):
     """
     for url in [RPM_REFERENCES_UPDATEINFO_URL, RPM_UNSIGNED_FIXTURE_URL]:
         repository, _ = init_and_sync(url=url)
-
-        assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-        assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+        content_summary = get_content_summary(repository)
+        assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+        assert content_summary["added"] == RPM_FIXTURE_SUMMARY
 
 
 @pytest.mark.parallel
 @pytest.mark.parametrize("policy", ["on_demand", "immediate"])
-def test_kickstart(policy, init_and_sync, rpm_content_distribution_trees_api):
+def test_kickstart(policy, init_and_sync, rpm_content_distribution_trees_api, get_content_summary):
     """Sync repositories with the rpm plugin.
 
     Do the following:
@@ -182,14 +173,14 @@ def test_kickstart(policy, init_and_sync, rpm_content_distribution_trees_api):
     6. Assert that the same number of packages are present.
     """
     repository, remote = init_and_sync(url=RPM_KICKSTART_FIXTURE_URL, policy=policy)
-
     latest_version_href = repository.latest_version_href
-    assert get_content_summary(repository.to_dict()) == RPM_KICKSTART_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_KICKSTART_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_KICKSTART_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_KICKSTART_FIXTURE_SUMMARY
 
     repository, _ = init_and_sync(repository=repository, remote=remote)
-
-    assert get_content_summary(repository.to_dict()) == RPM_KICKSTART_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_KICKSTART_FIXTURE_SUMMARY
     assert latest_version_href == repository.latest_version_href
 
     distribution_tree = rpm_content_distribution_trees_api.list(
@@ -198,7 +189,7 @@ def test_kickstart(policy, init_and_sync, rpm_content_distribution_trees_api):
     assert "RHEL" == distribution_tree.release_short
 
 
-def test_mutated_packages(init_and_sync):
+def test_mutated_packages(init_and_sync, get_content_summary, get_content):
     """Sync two copies of the same packages.
 
     Make sure we end up with only one copy.
@@ -216,9 +207,9 @@ def test_mutated_packages(init_and_sync):
     7. Assert that the packages have changed since the last sync.
     """
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL)
-
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_FIXTURE_SUMMARY
 
     # Save the copy of the original packages.
     original_packages = {
@@ -229,21 +220,18 @@ def test_mutated_packages(init_and_sync):
             content["release"],
             content["arch"],
         ): content
-        for content in get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]
+        for content in get_content(repository)["present"][RPM_PACKAGE_CONTENT_NAME]
     }
 
     # Create a remote with a different test fixture with the same NEVRA but
     # different digests.
     repository, _ = init_and_sync(repository=repository, url=RPM_SIGNED_FIXTURE_URL)
+    content_summary = get_content_summary(repository)
 
     # In case of "duplicates" the most recent one is chosen, so the old
     # package is removed from and the new one is added to a repo version.
-    assert (
-        len(get_added_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME])
-    ) == RPM_PACKAGE_COUNT
-    assert (
-        len(get_removed_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME])
-    ) == RPM_PACKAGE_COUNT
+    assert (content_summary["added"][RPM_PACKAGE_CONTENT_NAME]["count"]) == RPM_PACKAGE_COUNT
+    assert (content_summary["removed"][RPM_PACKAGE_CONTENT_NAME]["count"]) == RPM_PACKAGE_COUNT
 
     # Test that the packages have been modified.
     mutated_packages = {
@@ -254,14 +242,14 @@ def test_mutated_packages(init_and_sync):
             content["release"],
             content["arch"],
         ): content
-        for content in get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]
+        for content in get_content(repository)["present"][RPM_PACKAGE_CONTENT_NAME]
     }
 
     for nevra in original_packages:
         assert original_packages[nevra]["pkgId"] != mutated_packages[nevra]["pkgId"]
 
 
-def test_sync_diff_checksum_packages(init_and_sync):
+def test_sync_diff_checksum_packages(init_and_sync, get_content, get_content_summary):
     """Sync two fixture content with same NEVRA and different checksum.
 
     Make sure we end up with the most recently synced content.
@@ -277,9 +265,10 @@ def test_sync_diff_checksum_packages(init_and_sync):
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
 
     repository, _ = init_and_sync(repository=repository, url=RPM_SHA512_FIXTURE_URL)
+    content_in_repo = get_content(repository)
 
-    added_content = get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]
-    removed_content = get_removed_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]
+    added_content = content_in_repo["added"][RPM_PACKAGE_CONTENT_NAME]
+    removed_content = content_in_repo["removed"][RPM_PACKAGE_CONTENT_NAME]
 
     # In case of "duplicates" the most recent one is chosen, so the old
     # package is removed from and the new one is added to a repo version.
@@ -293,7 +282,7 @@ def test_sync_diff_checksum_packages(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_mutated_advisory_metadata(init_and_sync):
+def test_mutated_advisory_metadata(init_and_sync, get_content_summary, get_content):
     """Sync two copies of the same Advisory (only description is updated).
 
     Make sure we end up with only one copy.
@@ -312,25 +301,25 @@ def test_mutated_advisory_metadata(init_and_sync):
     7. Assert that the updaterecords have changed since the last sync.
     """
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
-
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert get_added_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+    assert content_summary["added"] == RPM_FIXTURE_SUMMARY
 
     original_updaterecords = {
         content["id"]: content
-        for content in get_content(repository.to_dict())[RPM_ADVISORY_CONTENT_NAME]
+        for content in get_content(repository)["present"][RPM_ADVISORY_CONTENT_NAME]
     }
 
     repository, _ = init_and_sync(repository=repository, url=RPM_UPDATED_UPDATEINFO_FIXTURE_URL)
-
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
-    assert len(get_added_content(repository.to_dict())[RPM_ADVISORY_CONTENT_NAME]) == 4
-    assert len(get_removed_content(repository.to_dict())[RPM_ADVISORY_CONTENT_NAME]) == 4
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
+    assert content_summary["added"][RPM_ADVISORY_CONTENT_NAME]["count"] == 4
+    assert content_summary["removed"][RPM_ADVISORY_CONTENT_NAME]["count"] == 4
 
     # Test that the updateinfo have been modified.
     mutated_updaterecords = {
         content["id"]: content
-        for content in get_content(repository.to_dict())[RPM_ADVISORY_CONTENT_NAME]
+        for content in get_content(repository)["present"][RPM_ADVISORY_CONTENT_NAME]
     }
 
     assert mutated_updaterecords != original_updaterecords
@@ -346,6 +335,9 @@ def test_optimize(
     rpm_repository_api,
     rpm_rpmremote_api,
     monitor_task,
+    get_content,
+    rpm_repository_factory,
+    rpm_rpmremote_factory,
 ):
     """Tests that sync is skipped when no critical parameters of the sync change.
 
@@ -377,7 +369,7 @@ def test_optimize(
 
     # create a new repo version, sync again, assert not optimized
     repository = rpm_repository_api.read(repository.pulp_href)
-    content = choice(get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME])
+    content = choice(get_content(repository)["present"][RPM_PACKAGE_CONTENT_NAME])
     response = rpm_repository_api.modify(
         repository.pulp_href, {"remove_content_units": [content["pulp_href"]]}
     )
@@ -401,7 +393,7 @@ def test_optimize(
     assert all(report.code != "sync.was_skipped" for report in task.progress_reports)
 
     # create new remote with the same URL and download_policy as the first and run a sync task
-    new_remote = gen_object_with_cleanup(rpm_rpmremote_api, gen_rpm_remote())
+    new_remote = rpm_rpmremote_factory()
     repository_sync_data = RpmRepositorySyncURL(
         remote=new_remote.pulp_href, sync_policy="mirror_content_only"
     )
@@ -451,7 +443,7 @@ def test_optimize(
 
 
 @pytest.mark.parallel
-def test_sync_advisory_new_version(init_and_sync):
+def test_sync_advisory_new_version(init_and_sync, get_content):
     """Sync a repository and re-sync with newer version of Advisory.
 
     Test if advisory with same ID and pkglist, but newer version is updated.
@@ -463,15 +455,16 @@ def test_sync_advisory_new_version(init_and_sync):
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL)
 
     repository, _ = init_and_sync(repository=repository, url=RPM_ADVISORY_UPDATED_VERSION_URL)
+    repo_content = get_content(repository)
 
     # check if newer version advisory was added and older removed
-    added_advisories = get_added_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+    added_advisories = repo_content["added"][PULP_TYPE_ADVISORY]
     added_advisory = [
         advisory["version"]
         for advisory in added_advisories
         if advisory["id"] == RPM_ADVISORY_TEST_ID
     ]
-    removed_advisories = get_removed_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+    removed_advisories = repo_content["removed"][PULP_TYPE_ADVISORY]
     removed_advisory = [
         advisory["version"]
         for advisory in removed_advisories
@@ -481,7 +474,7 @@ def test_sync_advisory_new_version(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_sync_advisory_old_version(init_and_sync):
+def test_sync_advisory_old_version(init_and_sync, get_content):
     """Sync a repository and re-sync with older version of Advisory.
 
     Test if advisory with same ID and pkglist, but older version is not updated.
@@ -496,7 +489,7 @@ def test_sync_advisory_old_version(init_and_sync):
     repository, _ = init_and_sync(repository=repository, url=RPM_UNSIGNED_FIXTURE_URL)
     repository_version_new = repository.latest_version_href
 
-    present_advisories = get_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+    present_advisories = get_content(repository)["present"][PULP_TYPE_ADVISORY]
     advisory_version = [
         advisory["version"]
         for advisory in present_advisories
@@ -511,22 +504,23 @@ def test_sync_advisory_old_version(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_sync_merge_advisories(init_and_sync):
+def test_sync_merge_advisories(init_and_sync, get_content):
     """Sync two advisories with same ID, version and different pkglist.
 
     Test if two advisories are merged.
     """
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL)
     repository, _ = init_and_sync(repository=repository, url=RPM_ADVISORY_DIFFERENT_PKGLIST_URL)
+    repo_content = get_content(repository)
 
     # check advisories were merged
-    added_advisories = get_added_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+    added_advisories = repo_content["added"][PULP_TYPE_ADVISORY]
     added_advisory_pkglist = [
         advisory["pkglist"]
         for advisory in added_advisories
         if advisory["id"] == RPM_ADVISORY_TEST_ID
     ]
-    removed_advisories = get_removed_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+    removed_advisories = repo_content["removed"][PULP_TYPE_ADVISORY]
     removed_advisory_pkglist = [
         advisory["pkglist"]
         for advisory in removed_advisories
@@ -569,7 +563,7 @@ def test_sync_advisory_diff_repo(init_and_sync):
         "ALLOW_AUTOMATIC_UNSAFE_ADVISORY_CONFLICT_RESOLUTION = True (q.v.) "
         "in your configuration. Advisory id: {}".format(RPM_ADVISORY_TEST_ID)
     )
-    assert error_msg in exc.value.task.to_dict()["error"]["description"]
+    assert error_msg in exc.value.task.error["description"]
 
 
 @pytest.mark.parallel
@@ -607,7 +601,7 @@ def test_sync_advisory_incomplete_pgk_list(init_and_sync):
         "At least one of the advisories is wrong. "
         "Advisory id: {}".format(RPM_ADVISORY_TEST_ID)
     )
-    assert error_msg in exc.value.task.to_dict()["error"]["description"]
+    assert error_msg in exc.value.task.error["description"]
 
 
 @pytest.mark.parallel
@@ -647,19 +641,21 @@ def test_sync_advisory_no_updated_date(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_sync_advisory_updated_update_date(init_and_sync):
+def test_sync_advisory_updated_update_date(init_and_sync, get_content):
     """Test sync advisory with updated update_date."""
     repository, _ = init_and_sync(url=RPM_UNSIGNED_FIXTURE_URL)
     repository, _ = init_and_sync(repository=repository, url=RPM_UPDATED_UPDATEINFO_FIXTURE_URL)
+    content_summary = get_content(repository)
 
+    # check advisories were merged
     added_advisory_date = [
         advisory["updated_date"]
-        for advisory in get_added_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+        for advisory in content_summary["added"][PULP_TYPE_ADVISORY]
         if RPM_ADVISORY_TEST_ID_NEW in advisory["id"]
     ]
     removed_advisory_date = [
         advisory["updated_date"]
-        for advisory in get_removed_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+        for advisory in content_summary["removed"][PULP_TYPE_ADVISORY]
         if RPM_ADVISORY_TEST_ID_NEW in advisory["id"]
     ]
 
@@ -667,24 +663,24 @@ def test_sync_advisory_updated_update_date(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_sync_advisory_older_update_date(init_and_sync):
+def test_sync_advisory_older_update_date(init_and_sync, get_content):
     """Test sync advisory with older update_date."""
     repository, _ = init_and_sync(url=RPM_UPDATED_UPDATEINFO_FIXTURE_URL)
+
     advisory_date = [
         advisory["updated_date"]
-        for advisory in get_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+        for advisory in get_content(repository)["present"][PULP_TYPE_ADVISORY]
         if advisory["id"] == RPM_ADVISORY_TEST_ID
     ]
 
     repository, _ = init_and_sync(repository, url=RPM_UNSIGNED_FIXTURE_URL)
+    repo_content = get_content(repository)
     advisory_date_new = [
         advisory["updated_date"]
-        for advisory in get_content(repository.to_dict())[PULP_TYPE_ADVISORY]
+        for advisory in repo_content["present"][PULP_TYPE_ADVISORY]
         if advisory["id"] == RPM_ADVISORY_TEST_ID
     ]
-    added_advisories = [
-        advisory["id"] for advisory in get_added_content(repository.to_dict())[PULP_TYPE_ADVISORY]
-    ]
+    added_advisories = [advisory["id"] for advisory in repo_content["added"][PULP_TYPE_ADVISORY]]
 
     # check if advisory is preserved and no advisory with same id was added
     assert parse_datetime(advisory_date[0]) == parse_datetime(advisory_date_new[0])
@@ -692,27 +688,28 @@ def test_sync_advisory_older_update_date(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_sync_repo_metadata_change(init_and_sync):
+def test_sync_repo_metadata_change(init_and_sync, get_content):
     """Sync RPM modular content."""
     repository, _ = init_and_sync(url=RPM_CUSTOM_REPO_METADATA_FIXTURE_URL)
     repository, _ = init_and_sync(
         repository=repository, url=RPM_CUSTOM_REPO_METADATA_CHANGED_FIXTURE_URL
     )
+    repo_content = get_content(repository)
 
     # Check if repository was updated with repository metadata
     assert repository.latest_version_href.rstrip("/")[-1] == "2"
-    assert PULP_TYPE_REPOMETADATA in get_added_content(repository.to_dict())
+    assert PULP_TYPE_REPOMETADATA in repo_content["added"]
 
 
 @pytest.mark.parallel
-def test_sync_modular_static_context(init_and_sync):
+def test_sync_modular_static_context(init_and_sync, get_content_summary, get_content):
     """Sync RPM modular content that includes the new static_context_field."""
     repository, _ = init_and_sync(url=RPM_MODULES_STATIC_CONTEXT_FIXTURE_URL)
+    content_summary = get_content_summary(repository)
+    summary = content_summary["present"]
+    added = content_summary["added"]
 
-    summary = get_content_summary(repository.to_dict())
-    added = get_added_content_summary(repository.to_dict())
-
-    modules = get_content(repository.to_dict())[PULP_TYPE_MODULEMD]
+    modules = get_content(repository)["present"][PULP_TYPE_MODULEMD]
     module_static_contexts = [
         (module["name"], module["version"]) for module in modules if module["static_context"]
     ]
@@ -723,14 +720,14 @@ def test_sync_modular_static_context(init_and_sync):
 
 @pytest.mark.parallel
 @pytest.mark.parametrize("sync_policy", ["mirror_content_only", "additive"])
-def test_sync_skip_srpm(init_and_sync, sync_policy):
+def test_sync_skip_srpm(init_and_sync, sync_policy, get_content):
     """In mirror_content_only mode, skip_types is allowed."""
     repository, _ = init_and_sync(
         url=SRPM_UNSIGNED_FIXTURE_URL, skip_types=["srpm"], sync_policy=sync_policy
     )
-
-    present_package_count = len(get_content(repository.to_dict())[PULP_TYPE_PACKAGE])
-    present_advisory_count = len(get_content(repository.to_dict())[PULP_TYPE_ADVISORY])
+    present_repo_content = get_content(repository)["present"]
+    present_package_count = len(present_repo_content[PULP_TYPE_PACKAGE])
+    present_advisory_count = len(present_repo_content[PULP_TYPE_ADVISORY])
     assert present_package_count == 0
     assert present_advisory_count == SRPM_UNSIGNED_FIXTURE_ADVISORY_COUNT
 
@@ -773,7 +770,7 @@ def test_invalid_url(init_and_sync):
     with pytest.raises(PulpTaskError) as exc:
         init_and_sync(url="http://i-am-an-invalid-url.com/invalid/")
 
-    assert exc.value.task.to_dict()["error"]["description"] is not None
+    assert exc.value.task.error["description"] is not None
 
 
 @pytest.mark.parallel
@@ -783,7 +780,7 @@ def test_invalid_rpm_content(init_and_sync):
         init_and_sync(url=RPM_INVALID_FIXTURE_URL)
 
     for key in ("missing", "filelists.xml"):
-        assert key in exc.value.task.to_dict()["error"]["description"]
+        assert key in exc.value.task.error["description"]
 
 
 @pytest.mark.parallel
@@ -804,7 +801,7 @@ def test_sync_metadata_with_unsupported_checksum_type(init_and_sync):
     assert (
         "does not contain at least one trusted hasher which "
         "is specified in the 'ALLOWED_CONTENT_CHECKSUMS'"
-    ) in exc.value.task.to_dict()["error"]["description"]
+    ) in exc.value.task.error["description"]
 
 
 @pytest.mark.parallel
@@ -828,7 +825,7 @@ def test_sync_packages_with_unsupported_checksum_type(init_and_sync):
     with pytest.raises(PulpTaskError) as exc:
         init_and_sync(url="https://fixtures.com/packages_with_unsupported_checksum")
 
-    error_description = exc.value.task.to_dict()["error"]["description"]
+    error_description = exc.value.task.error["description"]
     assert "rpm-with-md5/bear-4.1.noarch.rpm contains forbidden checksum type" in error_description
 
 
@@ -838,7 +835,7 @@ def test_complete_mirror_with_xml_base_fails(init_and_sync):
     with pytest.raises(PulpTaskError) as exc:
         init_and_sync(url=REPO_WITH_XML_BASE_URL, sync_policy="mirror_complete")
 
-    error_description = exc.value.task.to_dict()["error"]["description"]
+    error_description = exc.value.task.error["description"]
     assert "features which are incompatible with 'mirror' sync" in error_description
 
 
@@ -857,7 +854,7 @@ def test_complete_mirror_with_external_location_href_fails(init_and_sync):
             url="https://fixtures.com/repo_with_external_data", sync_policy="mirror_complete"
         )
 
-    error_description = exc.value.task.to_dict()["error"]["description"]
+    error_description = exc.value.task.error["description"]
     assert "features which are incompatible with 'mirror' sync" in error_description
 
 
@@ -875,7 +872,7 @@ def test_complete_mirror_with_delta_metadata_fails(init_and_sync):
         pass
         # init_and_sync(url=DRPM_UNSIGNED_FIXTURE_URL, sync_policy="mirror_complete")
 
-    error_description = exc.value.task.to_dict()["error"]["description"]
+    error_description = exc.value.task.error["description"]
     assert "features which are incompatible with 'mirror' sync" in error_description
 
 
@@ -884,14 +881,14 @@ def test_mirror_and_sync_policy_provided_simultaneously_fails(
     gen_object_with_cleanup,
     rpm_repository_api,
     rpm_rpmremote_api,
+    rpm_repository_factory,
+    rpm_rpmremote_factory,
 ):
     """
     Test that syncing fails if both the "mirror" and "sync_policy" params are provided.
     """
-    repository = gen_object_with_cleanup(rpm_repository_api, gen_repo())
-    remote = gen_object_with_cleanup(
-        rpm_rpmremote_api, gen_rpm_remote(url=RPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
-    )
+    repository = rpm_repository_factory()
+    remote = rpm_rpmremote_factory(url=RPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
 
     repository_sync_data = RpmRepositorySyncURL(
         remote=remote.pulp_href, sync_policy="mirror_complete", mirror=True
@@ -911,7 +908,7 @@ def test_sync_skip_srpm_fails_mirror_complete(init_and_sync):
 
 
 @pytest.mark.parallel
-def test_core_metadata(init_and_sync, rpm_package_api):
+def test_core_metadata(init_and_sync, rpm_package_api, get_content):
     """Test that the metadata returned by the Pulp API post-sync matches what we expect.
 
     Do the following:
@@ -925,11 +922,11 @@ def test_core_metadata(init_and_sync, rpm_package_api):
     package = rpm_package_api.list(
         name=RPM_COMPLEX_PACKAGE_DATA["name"], repository_version=repository.latest_version_href
     ).results[0]
-    package = package.to_dict()
+    package_dict = package.model_dump()
 
     # sort file and changelog metadata
-    package["changelogs"].sort(reverse=True)
-    for metadata in [package, RPM_COMPLEX_PACKAGE_DATA]:
+    package_dict["changelogs"].sort(reverse=True)
+    for metadata in [package_dict, RPM_COMPLEX_PACKAGE_DATA]:
         # the list-of-lists can't be sorted easily so we produce a string representation
         files = []
         for f in metadata["files"]:
@@ -942,14 +939,21 @@ def test_core_metadata(init_and_sync, rpm_package_api):
 
     # TODO: figure out how to un-ignore "time_file" without breaking the tests
     diff = dictdiffer.diff(
-        package,
+        package_dict,
         RPM_COMPLEX_PACKAGE_DATA,
-        ignore={"time_file", "pulp_created", "pulp_last_updated", "pulp_href", "prn"},
+        ignore={
+            "time_file",
+            "pulp_created",
+            "pulp_last_updated",
+            "pulp_href",
+            "prn",
+            "pulp_labels",
+        },
     )
     assert list(diff) == [], list(diff)
 
     # assert no package is marked modular
-    for pkg in get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]:
+    for pkg in get_content(repository)["present"][RPM_PACKAGE_CONTENT_NAME]:
         assert pkg["is_modular"] is False
 
 
@@ -969,8 +973,6 @@ def test_treeinfo_metadata(init_and_sync, rpm_content_distribution_trees_api):
         repository_version=repository.latest_version_href
     ).results[0]
     distribution_tree = distribution_tree.to_dict()
-    # delete pulp-specific metadata
-    distribution_tree.pop("pulp_href")
 
     # sort kickstart metadata so that we can compare the dicts properly
     for d in [distribution_tree, RPM_KICKSTART_DATA]:
@@ -992,6 +994,7 @@ def test_modular_metadata(
     rpm_modulemd_defaults_api,
     rpm_modulemd_obsoletes_api,
     delete_orphans_pre,
+    get_content,
 ):
     """Test that the metadata returned by the Pulp API post-sync matches what we expect.
 
@@ -1004,17 +1007,17 @@ def test_modular_metadata(
     repository, _ = init_and_sync(url=RPM_MODULAR_FIXTURE_URL, policy="on_demand")
 
     modules = [
-        md.to_dict()
+        md.model_dump()
         for md in rpm_modulemd_api.list(repository_version=repository.latest_version_href).results
     ]
     module_defaults = [
-        md.to_dict()
+        md.model_dump()
         for md in rpm_modulemd_defaults_api.list(
             repository_version=repository.latest_version_href
         ).results
     ]
     module_obsoletes = [
-        md.to_dict()
+        md.model_dump()
         for md in rpm_modulemd_obsoletes_api.list(
             repository_version=repository.latest_version_href
         ).results
@@ -1039,29 +1042,38 @@ def test_modular_metadata(
 
     for m1, m2 in zip(modules, RPM_MODULEMDS_DATA):
         diff = dictdiffer.diff(
-            m1, m2, ignore={"packages", "pulp_created", "pulp_last_updated", "pulp_href", "prn"}
+            m1,
+            m2,
+            ignore={
+                "packages",
+                "pulp_created",
+                "pulp_last_updated",
+                "pulp_href",
+                "prn",
+                "pulp_labels",
+            },
         )
         assert list(diff) == [], list(diff)
 
     for m1, m2 in zip(module_defaults, RPM_MODULEMD_DEFAULTS_DATA):
         diff = dictdiffer.diff(
-            m1, m2, ignore={"pulp_created", "pulp_last_updated", "pulp_href", "prn"}
+            m1, m2, ignore={"pulp_created", "pulp_last_updated", "pulp_href", "prn", "pulp_labels"}
         )
         assert list(diff) == [], list(diff)
 
     for m1, m2 in zip(module_obsoletes, RPM_MODULEMD_OBSOLETES_DATA):
         diff = dictdiffer.diff(
-            m1, m2, ignore={"pulp_created", "pulp_last_updated", "pulp_href", "prn"}
+            m1, m2, ignore={"pulp_created", "pulp_last_updated", "pulp_href", "prn", "pulp_labels"}
         )
         assert list(diff) == [], list(diff)
 
     # assert all package from modular repo is marked as modular
-    for pkg in get_content(repository.to_dict())[RPM_PACKAGE_CONTENT_NAME]:
+    for pkg in get_content(repository)["present"][RPM_PACKAGE_CONTENT_NAME]:
         assert pkg["is_modular"] is True
 
 
 @pytest.mark.parallel
-def test_additive_mode(init_and_sync):
+def test_additive_mode(init_and_sync, get_content):
     """Test of additive mode.
 
     1. Create repository, remote and sync it
@@ -1076,8 +1088,8 @@ def test_additive_mode(init_and_sync):
         sync_policy="additive",
     )
 
-    present_package_count = len(get_content(repository.to_dict())[PULP_TYPE_PACKAGE])
-    present_advisory_count = len(get_content(repository.to_dict())[PULP_TYPE_ADVISORY])
+    present_package_count = len(get_content(repository)["present"][PULP_TYPE_PACKAGE])
+    present_advisory_count = len(get_content(repository)["present"][PULP_TYPE_ADVISORY])
 
     assert (RPM_PACKAGE_COUNT + SRPM_UNSIGNED_FIXTURE_PACKAGE_COUNT) == present_package_count
     assert (RPM_ADVISORY_COUNT + SRPM_UNSIGNED_FIXTURE_ADVISORY_COUNT) == present_advisory_count
@@ -1085,7 +1097,7 @@ def test_additive_mode(init_and_sync):
 
 @pytest.mark.parallel
 @pytest.mark.parametrize("sync_policy", ["mirror_complete", "mirror_content_only"])
-def test_mirror_mode(sync_policy, init_and_sync, rpm_publication_api):
+def test_mirror_mode(sync_policy, init_and_sync, rpm_publication_api, get_content_summary):
     """Test of mirror mode."""
     repository, remote = init_and_sync(url=SRPM_UNSIGNED_FIXTURE_URL, policy="on_demand")
 
@@ -1098,7 +1110,8 @@ def test_mirror_mode(sync_policy, init_and_sync, rpm_publication_api):
 
     # check that one publication was created w/ no repository versions
     # and only the new content is present
-    assert get_content_summary(repository.to_dict()) == RPM_FIXTURE_SUMMARY
+    content_summary = get_content_summary(repository)
+    assert content_summary["present"] == RPM_FIXTURE_SUMMARY
     assert repository.latest_version_href == f"{repository.pulp_href}versions/2/"
 
     if sync_policy == "mirror_complete":
@@ -1110,9 +1123,9 @@ def test_mirror_mode(sync_policy, init_and_sync, rpm_publication_api):
 
 @pytest.mark.parallel
 def test_config_repo_mirror_sync(
+    distribution_base_url,
     rpm_rpmremote_factory,
     rpm_repository_factory,
-    rpm_publication_factory,
     rpm_distribution_factory,
     rpm_repository_api,
     rpm_publication_api,
@@ -1142,11 +1155,117 @@ def test_config_repo_mirror_sync(
         publication=publication.pulp_href, generate_repo_config=True
     )
 
-    content = requests.get(f"{distribution.base_url}config.repo").content
+    response = requests.get(f"{distribution_base_url(distribution.base_url)}/config.repo")
+    if not settings.CONTENT_ORIGIN:
+        assert response.status_code == 404
+    else:
+        content = response.content
+        # gpgcheck should follow repo_config
+        # repo_gpgcheck should match whether the upstream repo has signed metadata or not
+        assert bytes(f"[{distribution.name}]\n", "utf-8") in content
+        assert (
+            bytes(f"baseurl={distribution_base_url(distribution.base_url)}\n", "utf-8") in content
+        )
+        assert bytes("gpgcheck=1\n", "utf-8") in content
+        assert bytes("repo_gpgcheck=0", "utf-8") in content
 
-    # gpgcheck should follow repo_config
-    # repo_gpgcheck should match whether the upstream repo has signed metadata or not
-    assert bytes(f"[{distribution.name}]\n", "utf-8") in content
-    assert bytes(f"baseurl={distribution.base_url}\n", "utf-8") in content
-    assert bytes("gpgcheck=1\n", "utf-8") in content
-    assert bytes("repo_gpgcheck=0", "utf-8") in content
+
+@pytest.fixture
+def repo_4073_url(
+    tmp_path,
+    rpm_repository_factory,
+    rpm_rpmremote_factory,
+    rpm_publication_factory,
+    rpm_distribution_factory,
+    init_and_sync,
+):
+    REPODIR = tmp_path / "repo_4073"
+
+    def download_fixture(fixture_name: str, to_relative_path: str) -> Path:
+        url = urljoin(RPM_UNSIGNED_FIXTURE_URL, fixture_name)
+        response = requests.get(url)
+        response.raise_for_status()
+        pkg = REPODIR / to_relative_path
+        pkg.parent.mkdir(parents=True, exist_ok=True)
+        pkg.write_bytes(response.content)
+        return pkg
+
+    def get_package_list_from_repodata(repodata_dir: Path):
+        assert repodata_dir.exists()
+        primary_xml_file = next(repodata_dir.glob("*primary.xml*"))
+        result_repo = cr.RepositoryReader.from_metadata_files(str(primary_xml_file), None, None)
+        parsed = result_repo.parse_packages(only_primary=True)[0]  # {<checksum>: <cr.Package>}
+        return list(parsed.values())
+
+    # Setup repository packages layout
+    BEAR_FILENAME = "bear-4.1-1.noarch.rpm"
+    BEAR_RELATIVE_PATH = f"bear/{BEAR_FILENAME}"
+    CAMEL_FILENAME = "camel-0.1-1.noarch.rpm"
+    CAMEL_RELATIVE_PATH = f"camel/{BEAR_FILENAME}"  # yes, we're faking the filename
+
+    bear_rpm = download_fixture(BEAR_FILENAME, to_relative_path=BEAR_RELATIVE_PATH)
+    camel_rpm = download_fixture(CAMEL_FILENAME, to_relative_path=CAMEL_RELATIVE_PATH)
+    bear_pkg = cr.package_from_rpm(str(bear_rpm), location_href=BEAR_RELATIVE_PATH)
+    camel_pkg = cr.package_from_rpm(str(camel_rpm), location_href=CAMEL_RELATIVE_PATH)
+    BEAR_NAME = bear_pkg.name
+    CAMEL_NAME = camel_pkg.name
+
+    # Create repository with messed location_hrefs
+    with cr.RepositoryWriter(str(REPODIR), compression=cr.NO_COMPRESSION) as writer:
+        writer.set_num_of_pkgs(2)
+        writer.add_pkg(bear_pkg)
+        writer.add_pkg(camel_pkg)
+
+    # Assert it's what we expect
+    repodata_dir = REPODIR / "repodata"
+    packages_from_repodata = get_package_list_from_repodata(repodata_dir)
+    name_location_pkg_map = [(p.name, p.location_href) for p in packages_from_repodata]
+
+    assert (BEAR_NAME, BEAR_RELATIVE_PATH) in name_location_pkg_map
+    assert (CAMEL_NAME, CAMEL_RELATIVE_PATH) in name_location_pkg_map
+    return f"file://{REPODIR.absolute()}"
+
+
+def test_repo_4073(repo_4073_url):
+    assert repo_4073_url
+
+
+def test_repo_with_different_nevra_same_location_href(
+    repo_4073_url,
+    rpm_repository_factory,
+    rpm_rpmremote_factory,
+    init_and_sync,
+    rpm_publication_factory,
+    rpm_distribution_factory,
+    rpm_package_api,
+    distribution_base_url,
+    delete_orphans_pre,
+):
+    """Test syncing repository with packages having different NEVRA and same relative_path."""
+
+    # utils
+    def get_packages_in(repository) -> list[str]:
+        packages = rpm_package_api.list(repository_version=repository.latest_version_href)
+        package_name_location_href = [(pkg.name, pkg.location_href) for pkg in packages.results]
+        return package_name_location_href
+
+    def is_pkg_in_(distribution, pkg_relative_path) -> bool:
+        base_url = distribution_base_url(distribution.base_url)
+        pkg_url = urljoin(base_url, pkg_relative_path)
+        response = requests.get(pkg_url)
+        return response.status_code == 200
+
+    # when
+    REMOTE_URL = repo_4073_url
+    SYNC_POLICY = "mirror_content_only"
+    repository, _ = init_and_sync(url=REMOTE_URL, sync_policy=SYNC_POLICY)
+    rpm_publication_factory(repository=repository.pulp_href)
+    distribution = rpm_distribution_factory(repository=repository.pulp_href)
+
+    # then
+    package_name_location_href = get_packages_in(repository)
+    assert ("bear", "bear-4.1-1.noarch.rpm") in package_name_location_href
+    assert ("camel", "camel-0.1-1.noarch.rpm") in package_name_location_href
+
+    assert is_pkg_in_(distribution, "Packages/b/bear-4.1-1.noarch.rpm") is True
+    assert is_pkg_in_(distribution, "Packages/c/camel-0.1-1.noarch.rpm") is True
